from contextlib import contextmanager
from os import mkdir
from os.path import join
from tempfile import TemporaryDirectory


@contextmanager
def tmpdir(name=None):
    '''contextmanager for creating a temporary directory with an optional `name`'''
    if name:
        with TemporaryDirectory() as tmpdir:
            dir = join(tmpdir, name)
            mkdir(dir)
            yield dir
    else:
        yield TemporaryDirectory()



import re


def escape(*args):
    if len(args) == 1:
        arg = args[0]
        if isinstance(arg, dict):
            return " ".join(escape(k, v) for k,v in arg.items())
        else:
            backslashes = re.sub(r'\\',r'\\\\',str(arg))
            return re.sub(r'(["\n])',r'\\\1',backslashes)
    elif len(args) == 2:
        k,v = args
        return f'"{escape(k)}"="{escape(v)}"'





from git import Repo
from utz import contextmanager, git, run


@contextmanager
def txn(msg=None):
    '''Contextmanager that creates a synthetic merge with __enter__- and __exit__-time HEADs as parents, and tree (and
    message, by default) of the latter.'''
    prv = git.sha()
    yield
    cur = git.sha()
    if cur == prv:
        return

    if msg is None:
        msg = git.msg()
    elif isinstance(msg, str):
        pass
    elif callable(msg):
        msg = msg(prv, cur)
    else:
        raise ValueError(f'Invalid msg: {msg}')

    tree = Repo().tree().hexsha
    run('git','commit-tree',tree,'-p',prv,'-p',cur,'-m',msg)



from ..process import *


def exists(untracked=True, unstaged=True):
    lns = lines('git','status','--porcelain')
    if not untracked:
        lns = [ ln for ln in lns if not ln.startswith('??') ]
    if not unstaged:
        lns = [ ln for ln in lns if not ln.startswith(' ') ]
    return bool(lns)




from os import makedirs
from os.path import dirname
from pathlib import Path


def mkdir(path, *args, exist_ok=True, **kwargs):
    makedirs(str(path), *args, exist_ok=exist_ok, **kwargs)
    return path

def mkpar(path, *args, **kwargs):
    if isinstance(path, Path):
        path.parent.mkdir(exist_ok=True, parents=True)
    else:
        mkdir(dirname(path))
    return path
